@@ -16,7 +16,10 @@ EXTRA_CONFIGURE_OPTS=${EXTRA_CONFIGURE_OPTS-$DEFAULT_CONFIGURE_OPTS}
 #EXTRA_BUILD_OPTS
 #EXTRA_INSTALL_OPTS
 
-die () { printf "\nError during cabal-install bootstrap:\n$1\n" >&2 && exit 2 ;}
+die() {
+    printf "\nError during cabal-install bootstrap:\n%s\n" "$1" >&2
+    exit 2
+}
 
 # programs, you can override these by setting environment vars
 GHC="${GHC:-ghc}"
@@ -51,13 +54,12 @@ GZIP_PROGRAM="${GZIP_PROGRAM:-gzip}"
 SCOPE_OF_INSTALLATION="${SCOPE_OF_INSTALLATION:---user}"
 DEFAULT_PREFIX="${HOME}/.cabal"
 
-# Try to respect $TMPDIR.
-[ -"$TMPDIR"- = -""- ] &&
-  export TMPDIR=/tmp/cabal-$(echo $(od -XN4 -An /dev/random)) && mkdir $TMPDIR
+TMPDIR=$(mktemp -d -p /tmp -t cabal-XXXXXXX || mktemp -d -t cabal-XXXXXXX)
+export TMPDIR
 
 # Check for a C compiler, using user-set $CC, if any, first.
 for c in $CC gcc clang cc icc; do
-  $c --version 2>&1 >/dev/null && CC=$c &&
+  $c --version 1>/dev/null 2>&1 && CC=$c &&
   echo "Using $c for C compiler. If this is not what you want, set CC." >&2 &&
   break
 done
@@ -67,9 +69,15 @@ done
   If a C compiler is installed make sure it is on your PATH, or set $CC.'
 
 # Find the correct linker/linker-wrapper.
+#
+# See https://github.com/haskell/cabal/pull/4187#issuecomment-269074153.
 LINK="$(for link in collect2 ld; do
-          [ $($CC -print-prog-name=$link) = $link ] && continue ||
-          $CC -print-prog-name=$link
+          if [ $($CC -print-prog-name=$link) = $link ]
+          then
+              continue
+          else
+              $CC -print-prog-name=$link && break
+          fi
         done)"
 
 # Fall back to "ld"... might work.
@@ -158,9 +166,9 @@ while [ "$#" -gt 0 ]; do
   esac
 done
 
-# Do not try to use -j with GHC older than 7.8
+# Do not try to use -j with GHC 7.8 or older
 case $GHC_VER in
-    7.4*|7.6*)
+    7.4*|7.6*|7.8*)
         JOBS=""
         ;;
     *)
@@ -200,9 +208,9 @@ PREFIX=${PREFIX:-${DEFAULT_PREFIX}}
 
 # Versions of the packages to install.
 # The version regex says what existing installed versions are ok.
-PARSEC_VER="3.1.9";    PARSEC_VER_REGEXP="[3]\.[01]\."
+PARSEC_VER="3.1.11";    PARSEC_VER_REGEXP="[3]\.[01]\."
                        # >= 3.0 && < 3.2
-DEEPSEQ_VER="1.4.2.0"; DEEPSEQ_VER_REGEXP="1\.[1-9]\."
+DEEPSEQ_VER="1.4.3.0"; DEEPSEQ_VER_REGEXP="1\.[1-9]\."
                        # >= 1.1 && < 2
 
 case "$GHC_VER" in
@@ -213,35 +221,35 @@ case "$GHC_VER" in
         ;;
     *)
         # GHC >= 7.8
-        BINARY_VER="0.8.3.0"
+        BINARY_VER="0.8.5.1"
         BINARY_VER_REGEXP="[0]\.[78]\." # >= 0.7 && < 0.9
         ;;
 esac
 
 
-TEXT_VER="1.2.2.1";    TEXT_VER_REGEXP="((1\.[012]\.)|(0\.([2-9]|(1[0-1]))\.))"
+TEXT_VER="1.2.2.2";    TEXT_VER_REGEXP="((1\.[012]\.)|(0\.([2-9]|(1[0-1]))\.))"
                        # >= 0.2 && < 1.3
-NETWORK_VER="2.6.2.1"; NETWORK_VER_REGEXP="2\.[0-6]\."
+NETWORK_VER="2.6.3.2"; NETWORK_VER_REGEXP="2\.[0-6]\."
                        # >= 2.0 && < 2.7
 NETWORK_URI_VER="2.6.1.0"; NETWORK_URI_VER_REGEXP="2\.6\."
                        # >= 2.6 && < 2.7
-CABAL_VER="1.24.1.0";  CABAL_VER_REGEXP="1\.24\.[0-9]"
-                       # >= 1.24 && < 1.25
-TRANS_VER="0.5.2.0";   TRANS_VER_REGEXP="0\.[45]\."
+CABAL_VER="2.0.1.0";  CABAL_VER_REGEXP="2\.0\.[0-9]"
+                       # >= 2.0 && < 2.1
+TRANS_VER="0.5.4.0";   TRANS_VER_REGEXP="0\.[45]\."
                        # >= 0.2.* && < 0.6
 MTL_VER="2.2.1";       MTL_VER_REGEXP="[2]\."
                        #  >= 2.0 && < 3
-HTTP_VER="4000.3.3";   HTTP_VER_REGEXP="4000\.(2\.([5-9]|1[0-9]|2[0-9])|3\.?)"
+HTTP_VER="4000.3.7";   HTTP_VER_REGEXP="4000\.(2\.([5-9]|1[0-9]|2[0-9])|3\.?)"
                        # >= 4000.2.5 < 4000.4
-ZLIB_VER="0.6.1.1";    ZLIB_VER_REGEXP="(0\.5\.([3-9]|1[0-9])|0\.6)"
+ZLIB_VER="0.6.1.2";    ZLIB_VER_REGEXP="(0\.5\.([3-9]|1[0-9])|0\.6)"
                        # >= 0.5.3 && <= 0.7
-TIME_VER="1.6"         TIME_VER_REGEXP="1\.[1-6]\.?"
-                       # >= 1.1 && < 1.7
+TIME_VER="1.8.0.2"     TIME_VER_REGEXP="1\.[1-8]\.?"
+                       # >= 1.1 && < 1.9
 RANDOM_VER="1.1"       RANDOM_VER_REGEXP="1\.[01]\.?"
                        # >= 1 && < 1.2
 STM_VER="2.4.4.1";     STM_VER_REGEXP="2\."
                        # == 2.*
-ASYNC_VER="2.1.0";     ASYNC_VER_REGEXP="2\."
+ASYNC_VER="2.1.1.1";   ASYNC_VER_REGEXP="2\."
                        # 2.*
 OLD_TIME_VER="1.1.0.3"; OLD_TIME_VER_REGEXP="1\.[01]\.?"
                        # >=1.0.0.0 && <1.2
@@ -249,10 +257,16 @@ OLD_LOCALE_VER="1.0.0.7"; OLD_LOCALE_VER_REGEXP="1\.0\.?"
                        # >=1.0.0.0 && <1.1
 BASE16_BYTESTRING_VER="0.1.1.6"; BASE16_BYTESTRING_VER_REGEXP="0\.1"
                        # 0.1.*
-BASE64_BYTESTRING_VER="1.0.0.1"; BASE64_BYTESTRING_REGEXP="1\."
+BASE64_BYTESTRING_VER="1.0.0.1"; BASE64_BYTESTRING_VER_REGEXP="1\."
                        # >=1.0
-CRYPTOHASH_SHA256_VER="0.11.7.1"; CRYPTOHASH_SHA256_VER_REGEXP="0\.11\.?"
+CRYPTOHASH_SHA256_VER="0.11.100.1"; CRYPTOHASH_SHA256_VER_REGEXP="0\.11\.?"
                        # 0.11.*
+MINTTY_VER="0.1";      MINTTY_VER_REGEXP="0\.1\.?"
+                       # 0.1.*
+ECHO_VER="0.1.3";      ECHO_VER_REGEXP="0\.1\.[3-9]"
+                       # >= 0.1.3 && < 0.2
+EDIT_DISTANCE_VER="0.2.2.1"; EDIT_DISTANCE_VER_REGEXP="0\.2\.2\.?"
+                       # 0.2.2.*
 ED25519_VER="0.0.5.0"; ED25519_VER_REGEXP="0\.0\.?"
                        # 0.0.*
 HACKAGE_SECURITY_VER="0.5.2.2"; HACKAGE_SECURITY_VER_REGEXP="0\.5\.(2\.[2-9]|[3-9])"
@@ -260,7 +274,7 @@ HACKAGE_SECURITY_VER="0.5.2.2"; HACKAGE_SECURITY_VER_REGEXP="0\.5\.(2\.[2-9]|[3-
 BYTESTRING_BUILDER_VER="0.10.8.1.0"; BYTESTRING_BUILDER_VER_REGEXP="0\.10\.?"
 TAR_VER="0.5.0.3";     TAR_VER_REGEXP="0\.5\.([1-9]|1[0-9]|0\.[3-9]|0\.1[0-9])\.?"
                        # >= 0.5.0.3  && < 0.6
-HASHABLE_VER="1.2.4.0"; HASHABLE_VER_REGEXP="1\."
+HASHABLE_VER="1.2.6.1"; HASHABLE_VER_REGEXP="1\."
                        # 1.*
 
 HACKAGE_URL="https://hackage.haskell.org/package"
@@ -354,7 +368,7 @@ install_pkg () {
   [ -x Setup ] && ./Setup clean
   [ -f Setup ] && rm Setup
 
-  ${GHC} --make ${JOBS} Setup -o Setup ||
+  ${GHC} --make ${JOBS} Setup -o Setup -XRank2Types -XFlexibleContexts ||
     die "Compiling the Setup script failed."
 
   [ -x Setup ] || die "The Setup script does not exist or cannot be run"
@@ -399,10 +413,8 @@ do_pkg () {
         echo "Downloading ${PKG}-${VER}..."
         fetch_pkg ${PKG} ${VER}
     fi
-    unpack_pkg ${PKG} ${VER}
-    cd "${PKG}-${VER}"
-    install_pkg ${PKG} ${VER}
-    cd ..
+    unpack_pkg "${PKG}" "${VER}"
+    (cd "${PKG}-${VER}" && install_pkg ${PKG} ${VER})
   fi
 }
 
@@ -414,11 +426,9 @@ do_Cabal_pkg () {
         if need_pkg "Cabal" ${CABAL_VER_REGEXP}
         then
             echo "Cabal-${CABAL_VER} will be installed from the local Git clone."
-            cd ../Cabal
-            install_pkg ${CABAL_VER} ${CABAL_VER_REGEXP}
-            cd ../cabal-install
+            (cd ../Cabal && install_pkg ${CABAL_VER} ${CABAL_VER_REGEXP})
         else
-            echo "Cabal-${CABAL_VER} is already installed and the version is ok."
+            echo "Cabal is already installed and the version is ok."
         fi
     else
         info_pkg "Cabal"        ${CABAL_VER}   ${CABAL_VER_REGEXP}
@@ -481,6 +491,9 @@ info_pkg "base64-bytestring" ${BASE64_BYTESTRING_VER} \
     ${BASE64_BYTESTRING_VER_REGEXP}
 info_pkg "cryptohash-sha256" ${CRYPTOHASH_SHA256_VER} \
     ${CRYPTOHASH_SHA256_VER_REGEXP}
+info_pkg "mintty"        ${MINTTY_VER}        ${MINTTY_VER_REGEXP}
+info_pkg "echo"          ${ECHO_VER}          ${ECHO_VER_REGEXP}
+info_pkg "edit-distance" ${EDIT_DISTANCE_VER} ${EDIT_DISTANCE_VER_REGEXP}
 info_pkg "ed25519"           ${ED25519_VER}          ${ED25519_VER_REGEXP}
 info_pkg "tar"               ${TAR_VER}              ${TAR_VER_REGEXP}
 info_pkg "hashable"          ${HASHABLE_VER}         ${HASHABLE_VER_REGEXP}
@@ -516,6 +529,9 @@ do_pkg   "base64-bytestring" ${BASE64_BYTESTRING_VER} \
     ${BASE64_BYTESTRING_VER_REGEXP}
 do_pkg   "cryptohash-sha256" ${CRYPTOHASH_SHA256_VER} \
     ${CRYPTOHASH_SHA256_VER_REGEXP}
+do_pkg "mintty"        ${MINTTY_VER}        ${MINTTY_VER_REGEXP}
+do_pkg "echo"          ${ECHO_VER}          ${ECHO_VER_REGEXP}
+do_pkg "edit-distance" ${EDIT_DISTANCE_VER} ${EDIT_DISTANCE_VER_REGEXP}
 do_pkg   "ed25519"           ${ED25519_VER}          ${ED25519_VER_REGEXP}
 
 # We conditionally install bytestring-builder, depending on the bytestring
